%
%
%  RGB to Gray for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="RGB to Gray" icon="Matlab" tooltip="Add new channel which intensity is given by a weighted sum of the intensity of the other channels.">
%          <Command>MatlabXT::XTRGB2Gray(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%
%   Add a gray channel to the dataset. 
%   Its intensity is given by a weighted sum of the intensity of the other
%       channels.
%   The coefficients of the sum are given by the user.
%
%
function XTRGB2Gray(aImarisApplicationID,aChannelSelection)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

%Get the dataset
vDataSet = vImarisApplication.GetDataSet.Clone;

if nargin > 1
  % convert channel selection string into array
  vChannelSelection = str2num(aChannelSelection)
else
  vChannelSelection = 1:vDataSet.GetSizeC;
  for vIndex = 1:vDataSet.GetSizeC
    vChannelSelection(vIndex) = vImarisApplication.GetChannelVisibility(vIndex - 1);
  end
end

% Draw the initial dialog with values such as their sum is equal to 1
vNumberOfChannelsSelected = sum(vChannelSelection);
if vNumberOfChannelsSelected < 1
  return
end

% Compute the initial weight values
vDefaultWeigth = (1/vNumberOfChannelsSelected);

vPrompt = cell(vNumberOfChannelsSelected-1, 1);
vDefaultPromptAnswer = cell(vNumberOfChannelsSelected-1, 1);
for vIndex = 1:(vNumberOfChannelsSelected-1)
    vPrompt{vIndex} = ['Channel ' int2str(vIndex)];
    vDefaultPromptAnswer{vIndex} = num2str(vDefaultWeigth);
end

vPrompt(vNumberOfChannelsSelected) = {['Channel ' int2str(vNumberOfChannelsSelected)]};
vDefaultPromptAnswer(vNumberOfChannelsSelected) = {num2str(1 - (vNumberOfChannelsSelected-1)*vDefaultWeigth)};

% Draw the dialog
vAnswer = inputdlg(vPrompt,'RGB2Gray',1,vDefaultPromptAnswer);

% Do something only if values exist
if isempty(vAnswer)
  return
end

vDefineWeight = str2num(char(vAnswer));
% the sum of the defined weigths must be equal to 1
if sum(vDefineWeight) < 0.9 || sum(vDefineWeight) > 1.1
  msgbox('Sum of weights must be 1')
  return
end

% Dataset properties
vNumberOfTimePoints = vDataSet.GetSizeT;
vNumberOfChannels = vDataSet.GetSizeC;
vNumberOfSlices = vDataSet.GetSizeZ;
vSizeXY = [vDataSet.GetSizeX, vDataSet.GetSizeY];

vTotalCount = vNumberOfSlices*vNumberOfChannelsSelected*vNumberOfTimePoints;

vProgressDisplay = waitbar(0,'XTRGB2Gray:Processing Volume...');

vCount = 0;
% Create a new channel
vDataSet.SetSizeC(vNumberOfChannels + 1);            
vDataSet.SetChannelName(vNumberOfChannels, 'Gray Channel');

% Loop through the dataset...
for vTime = 1:vNumberOfTimePoints
    for vSlice = 1:vNumberOfSlices
        vImageSliceSum = zeros(vSizeXY);
        for vChannel = 1:vNumberOfChannels
            if vChannelSelection(vChannel) == 1
                % Add each channel of one slice and one
                % timepoint with the defined weights. The
                % resulting image is the gray image.
                vImageSlice = vDataSet.GetDataSliceFloats(vSlice-1, vChannel-1, vTime-1);
                vImageSlice = double(vImageSlice) .* vDefineWeight(vChannel);                                       
                vImageSliceSum = vImageSliceSum + vImageSlice;
                vCount = vCount + 1; 
            end                        
        end

        % The image is given back to Imaris
        if strcmp(vDataSet.GetType,'eTypeUInt8')
          vDataSet.SetDataSliceBytes(uint8(vImageSliceSum), vSlice-1, vNumberOfChannels, vTime-1);
        elseif strcmp(vDataSet.GetType,'eTypeUInt16')
          vDataSet.SetDataSliceShorts(uint16(vImageSliceSum), vSlice-1, vNumberOfChannels, vTime-1);
        elseif strcmp(vDataSet.GetType,'eTypeFloat')
          vDataSet.SetDataSliceFloats(single(vImageSliceSum), vSlice-1, vNumberOfChannels, vTime-1);
        end

        waitbar(vCount/vTotalCount)
    end
end             
close(vProgressDisplay)
vImarisApplication.SetDataSet(vDataSet);


